﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.ComponentModel; // INotifyPropertyChanged 인터페이스가 구현된 네임스페이스

namespace WithoutBinding {

	public class Person : INotifyPropertyChanged {
		// INotifyPropertyChanged 인터페이스 멤버들
		public event PropertyChangedEventHandler PropertyChanged;
		
		protected void Notify(string propName) {
			if (this.PropertyChanged != null) {
				PropertyChanged(this, new PropertyChangedEventArgs(propName));
			}
		}

		string name;
		public string Name {
			get { return this.name; }
			set {
				if (this.name == value) { return; }
				this.name = value;
				Notify("Name");
			}
		}

		int age;
		public int Age {
			get { return this.age; }
			set {
				if (this.age == value) { return; }
				this.age = value;
				Notify("Age");
			}
		}

		public Person() { }
		public Person(string name, int age) {
			this.name = name;
			this.age = age;
		}
	}

	public partial class Window1 : Window {
		Person person = new Person("웹지니", 11);

		public Window1() {
			InitializeComponent();

			// 기본 값을 채운다.
			this.nameTextBox.Text = person.Name;
			this.ageTextBox.Text = person.Age.ToString();

			// Person 객체의 속성이 변하는 지를 감시하기 위해 이벤트 핸들러를 등록
			person.PropertyChanged += person_PropertyChanged;

			// 컨트롤에 입력된 값이 변경되는지를 감시하기 위해 이벤트 핸들러를 등록
			this.nameTextBox.TextChanged += nameTextBox_TextChanged;
			this.ageTextBox.TextChanged += ageTextBox_TextChanged;

			// 생일 버튼의 Click 이벤트 핸들러를 등록한다.
			this.birthdayButton.Click += birthdayButton_Click;
		}

		void person_PropertyChanged(object sender, PropertyChangedEventArgs e) {

			switch (e.PropertyName) {
				case "Name":
					this.nameTextBox.Text = person.Name;
					break;

				case "Age":
					this.ageTextBox.Text = person.Age.ToString();
					break;
			}
		}

		void nameTextBox_TextChanged(object sender, TextChangedEventArgs e) {
			person.Name = nameTextBox.Text;
		}

		void ageTextBox_TextChanged(object sender, TextChangedEventArgs e) {
			int age = 0;
			if (int.TryParse(ageTextBox.Text, out age)) {
				person.Age = age;
			}
		}

		void birthdayButton_Click(object sender, RoutedEventArgs e) {
			++person.Age;
			MessageBox.Show(
			  string.Format(
				"생일 축하해요, {0}!, 현재 나이는 {1}살!",
				person.Name,
				person.Age),
			  "생일");
		}

	}
}
